///////////////////////////////////////////////////////////////////////////////
//
// GameAppUi.cpp
//
///////////////////////////////////////////////////////////////////////////////
#include "GameAppUi.h"
#include "GameAppDoc.h"
#include "GameAppView.h"
#include "GameResources.h"
#include "GameResourceUtils.h"
#include "Game.pan"

#include "Screen.h"
#include "ScreenFurniture.h"
#include "ObjectController.h"
#include "AsteroidState.h"
#include "PlayerStates.h"

#include <e32keys.h>
#include <barsread.h>
#include <eikenv.h>
#include <eikmenup.h>
#include <eiktbar.h>
#include <eikcmbut.h>
#include <uikon.hrh>
#include <eikfnlab.h>
#include <eikrutil.h>
#include <eikon.rsg>
#include <uikon.hrh>

const TInt KMaxTextBufLen = 300;

///////////////////////////////////////////////////////////////////////////////
// CPowerupScreenBox
///////////////////////////////////////////////////////////////////////////////
class CPowerupScreenBox : public CScreenSpriteBox
	{
public:
	CPowerupScreenBox(CScreenBitmap& aScreen, CGameSpriteTable&);
	void ConstructL(TSpriteId aSpriteId, const TFontSpec& aFont);
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
CPowerupScreenBox::CPowerupScreenBox(CScreenBitmap& aScreen, CGameSpriteTable& aSpriteTable)
: CScreenSpriteBox(aScreen, aSpriteTable)
	{
	}

///////////////////////////////////////////////////////////////////////////////
void CPowerupScreenBox::ConstructL(TSpriteId aSpriteId, const TFontSpec& aFont)
	{
	RGameSprite sprite(*iSprite.SpriteTable());
	sprite.OpenL(aSpriteId);
	TRect r(sprite.FrameSize());
	sprite.Close();
	CScreenNumberBox* textBox = new(ELeave) CScreenNumberBox(iScreen);
	CleanupStack::PushL(textBox);
	textBox->ConstructL(r, aFont);
	CScreenSpriteBox::ConstructL(r.iTl, aSpriteId, textBox);
	CleanupStack::Pop(textBox);
	r.iBr.iX += r.Width();
	SetRectL(r);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// CGameAppUi
///////////////////////////////////////////////////////////////////////////////
CGameAppUi::CGameAppUi()
: iTextBoxIndex(-1), iHighScoreBoxIndex(-1), iPowerupsBoxIndex(-1)
	{
	}

///////////////////////////////////////////////////////////////////////////////
CGameAppUi::~CGameAppUi()
	{
	if(iAsyncCommand)
		{
		iAsyncCommand->Cancel();
		delete iAsyncCommand;
		}
	if(iView)
		{
		RemoveFromStack(iView);
		delete iView;
		}
	delete iText;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::ConstructL()
	{
	BaseConstructL(CEikAppUi::ENoScreenFurniture);
	iView = new(ELeave) CGameAppView(GameDoc());
	iView->ConstructL(ClientRect());
	iView->ActivateL();
	AddToStackL(iView);
	iText = HBufC::NewL(KMaxTextBufLen);
	HandleModelChangeL();
	CmdPlayAsteroidsL();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::HandleModelChangeL()
	{
	CGameModel& model = GameDoc().Model();
	iEikonEnv->UpdateTaskNameL();
	if(iView)
		{
		model.SetGameObserver(*this);
		model.SetPlayerInput(iView);
		}
	TFontSpec fontSpec;
	fontSpec.iTypeface.SetIsProportional(EFalse);
	fontSpec.iTypeface.SetIsSerif(EFalse);
	// Score / Wave text box.
	const TSize s(model.Screen().Size());
	TRect r;
	TInt w = s.iWidth / 2;
	TInt h = s.iHeight / 10;
	r.iTl.iX = 5;
	r.iTl.iY = 5;
	r.iBr.iX = r.iTl.iX + w;
	r.iBr.iY = r.iTl.iY + h;
	CScreenTextBox* box = new(ELeave) CScreenTextBox(model.Screen());
	CleanupStack::PushL(box);
	TResourceReader reader;
	iEikonEnv->CreateResourceReaderLC(reader, R_ASTEROIDS_TEXTBOX);
	box->InternalizeL(reader);
	CleanupStack::PopAndDestroy(); // resource
	box->SetRectL(r);
	box->SetFontL(fontSpec);
	iTextBoxIndex = model.AddScreenBoxL(box);
	CleanupStack::Pop(box);
	// High Scores text box.
	w = s.iWidth / 4;
	h = s.iHeight / 10;
	r.iTl.iX = w;
	r.iTl.iY = h;
	r.iBr.iX = r.iTl.iX + (w * 2);
	r.iBr.iY = r.iTl.iY + (h * 8);
	box = new(ELeave) CScreenMultilineTextBox(model.Screen());
	CleanupStack::PushL(box);
	iEikonEnv->CreateResourceReaderLC(reader, R_ASTEROIDS_HIGHSCORE_TEXTBOX);
	box->InternalizeL(reader);
	CleanupStack::PopAndDestroy(); // resource
	box->SetRectL(r);
	box->SetFontL(fontSpec);
	iHighScoreBoxIndex = model.AddScreenBoxL(box);
	CleanupStack::Pop(box);
	// Powerups box.
	CScreenBoxList* powerupList = new(ELeave) CScreenBoxList(model.Screen());
	CleanupStack::PushL(powerupList);
	powerupList->ConstructL(TPoint());
	iEikonEnv->CreateResourceReaderLC(reader, R_ASTEROIDS_POWERUPBOX);
	powerupList->InternalizeL(reader);
	CleanupStack::PopAndDestroy(); // resource
	iPowerupsBoxIndex = model.AddScreenBoxL(powerupList);
	CleanupStack::Pop(powerupList);

	iEikonEnv->CreateResourceReaderLC(reader, R_ASTEROIDS_POWERUP_ROWSIZE);
	iPowerupsBoxRowSize = GameResourceUtils::ReadTSize(reader);
	CleanupStack::PopAndDestroy(); // resource
	iEikonEnv->CreateResourceReaderLC(reader, R_ASTEROIDS_POWERUP_PENSTYLE);
	iPowerupsBoxRowPenStyle.InternalizeL(reader);
	CleanupStack::PopAndDestroy(); // resource
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::HandleCommandL(TInt aCommand)
	{
	if(!IsCommandAvailable(aCommand))
		{
		iEikonEnv->InfoMsg(R_EIK_TBUF_NOT_AVAILABLE);
		return;
		}
	StopGame();
	switch(aCommand)
		{
	case EEikCmdExit:
		Exit();
		break;

	case ECmdPlay:
		CmdPlayAsteroidsL();
		break;

	case ECmdPause:
		CmdPauseResumeL();
		break;

	default:
		break;
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::OnGameEventL(CGameModel&, MGameModelObserver::TNotification aNotification)
	{
	switch(aNotification)
		{
	case MGameModelObserver::EGameTick:
		UpdateGameStateL();
		DrawScreen();
		break;
	case MGameModelObserver::EStartGame:
		FirstWaveL();
		break;
	case MGameModelObserver::EStopGame:
	case MGameModelObserver::EResumeGame:
	default:
		break;
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::DynInitMenuPaneL(TInt aResourceId, CEikMenuPane* aMenuPane)
	{
	if(aResourceId == R_THRUST_GAME_MENU)
		{
		TInt commandId = 0;
		TInt c = aMenuPane->NumberOfItemsInPane();
		while(c--)
			{
			commandId = aMenuPane->MenuItemCommandId(c);
			if(commandId)
				{
				const TBool available = IsCommandAvailable(commandId);
				aMenuPane->SetItemDimmed(commandId, !available);
				if((available) && (commandId == ECmdPause))
					{
					aMenuPane->SetItemTextL(ECmdPause, (iFlags & EStatePaused) ? (R_THRUST_MENUTEXT_RESUME) : (R_THRUST_MENUTEXT_PAUSE));
					}
				}
			}
		}
	CEikAppUi::DynInitMenuPaneL(aResourceId, aMenuPane);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::HandleForegroundEventL(TBool aForeground)
	{
	CEikAppUi::HandleForegroundEventL(aForeground);
	DoHandleForegroundEventL(aForeground);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::DoHandleForegroundEventL(TBool aForeground)
	{
	if(aForeground)
		{
		if(iFlags & EStateSuspended)
			{
			ResumeGameL();
			iFlags &= ~EStateSuspended;
			}
		}
	else if(GameDoc().Model().IsActive())
		{
		iFlags |= EStateSuspended;
		StopGame();
		}
	}

///////////////////////////////////////////////////////////////////////////////
CGameDocument& CGameAppUi::GameDoc() const
	{
	__ASSERT_DEBUG(iDocument, Panic(EPanicAppUiNotConstructed));
	return STATIC_CAST(CGameDocument&, *iDocument);
	}

///////////////////////////////////////////////////////////////////////////////
TBool CGameAppUi::IsCommandAvailable(TInt aCommand) const
	{
	TBool ret = EFalse;
	switch(aCommand)
		{
	case EEikCmdExit:
	case ECmdPlay:
		ret = ETrue;
		break;

	case ECmdPause:
		ret = (GameDoc().Model().IsActive()) || (iFlags & EStatePaused);
		break;

	default:
		break;
		}
	return ret;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::CmdPlayAsteroidsL()
	{
	const TInt KBusyDelay = 1000000; // 1 sec
	_LIT(KNewGameMsg,"Starting new game...");
	iEikonEnv->BusyMsgL(KNewGameMsg, TTimeIntervalMicroSeconds32(KBusyDelay));
	CleanupStack::PushL(TCleanupItem(CleanupRemoveBusy));
	GameDoc().NewGameL();
	StartGameL();
	CleanupStack::PopAndDestroy(); // BusyMsg
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::CmdPauseResumeL()
	{
	if(iFlags & EStatePaused)
		{
		ResumeGameL();
		iFlags &= ~EStatePaused;
		iEikonEnv->BusyMsgCancel();
		}
	else
		{
		StopGame();
		iFlags |= EStatePaused;
		_LIT(KPausedText,"Paused. [ctrl+p] to resume");
		iEikonEnv->BusyMsgL(KPausedText, TGulAlignment(EHCenterVCenter));
		}
	}


///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::StartGameL()
	{
	if(iFlags & EStatePaused)
		{
		iEikonEnv->BusyMsgCancel();
		iFlags &= ~EStatePaused;
		}
	iFlags &= ~EStateSuspended;
	CGameModel& model = GameDoc().Model();
	model.SetGameObserver(*this);
	model.PlayL();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::StopGame()
	{
	GameDoc().Model().Stop();
	iView->ResetPlayerAddress();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::ResumeGameL()
	{
	CGameModel& model = GameDoc().Model();
	model.ResumeL();
	}

///////////////////////////////////////////////////////////////////////////////
CGameObject* CGameAppUi::PlayerL()
	{
	CGameModel& model = GameDoc().Model();
	CGameObject* o = model.Objects().ObjectL(iView->PlayerAddressL(model));
	if(o)
		{
		if(o->StateData().CurrentState() != KGameObjType1_ObjectPlayerState)
			{
			iView->ResetPlayerAddress();
			o = model.Objects().ObjectL(iView->PlayerAddressL(model));
			}
		}
	return o;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::DrawScreen()
	{
	iView->RedrawScreen();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::UpdatePickupsL()
	{
	CGameObject* player = PlayerL();
	if(player)
		{
		CGameModel& model = GameDoc().Model();
		CScreenBoxList& box = static_cast<CScreenBoxList&>(model.ScreenBox(iPowerupsBoxIndex));
		box.Reset();
		const TFontSpec font(iEikonEnv->NormalFont()->FontSpecInTwips());
		const RArray<TMessageCounter>& counters = player->StateData().Counters();
		TInt numCounters = counters.Count();
		for(TInt c=0; c<numCounters; c++)
			{
			const CGameObjectPlayerPickupState* state = static_cast<const CGameObjectPlayerPickupState*>(model.Objects().StateL(counters[c].iExpiryMessage.iState));
			if(state)
				{
				CPowerupScreenBox* powerup = new(ELeave) CPowerupScreenBox(model.Screen(), model.Sprites());
				CleanupStack::PushL(powerup);
				powerup->ConstructL(state->SpriteId(), font);
				powerup->SetFillStyle(box.FillStyle());
				box.AddBoxL(powerup, iPowerupsBoxRowSize);
				CleanupStack::Pop(powerup);
				CScreenNumberBox& numBox = static_cast<CScreenNumberBox&>(powerup->TextBox());
				numBox.SetFillStyle(box.FillStyle());
				numBox.SetPenStyle(iPowerupsBoxRowPenStyle);
				numBox.SetFontL(font);
				numBox.SetNumberL(TUint(counters[c].iCount));
				}
			}
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::UpdateGameStateL()
	{
	CGameObject* player = PlayerL();
	CGameModel& model = GameDoc().Model();
	if(player == NULL)
		{
		if(iFlags & EStatePlayerDead)
			{
			MGameModelPlayerInput::TPlayerKeyState input;
			iView->GetPlayerInput(model, input);
			if(input.IsPressed(EPlayerKeyBegin))
				{
				AsyncCommandL(ECmdPlay);
				SaveHighScores();
				}
			else if(input.IsPressed(EPlayerKeyEsc))
				{
				AsyncCommandL(EEikCmdExit);
				SaveHighScores();
				}
			else if(iCurrentTextBoxLine >= 0)
				{
				const TChar key(iView->GetKeyChar());
				UpdateHighScoreEditL(key);
				}
			}
		else
			{
			// Game over.
			GameOverL();
			}
		}
	else
		{
		CArrayPtrFlat<CGameObject>* enemies = model.Objects().ObjectsByAllegienceLC(player->Allegience(), CGameObjectController::EExclude);
		TInt numBadGuys = enemies->Count();
		if(numBadGuys <= 10) // Optimisation. If there are more than 10 objects, don't bother checking for end of level.
			{
			while(numBadGuys--)
				{
				if(enemies->At(numBadGuys)->Allegience() == KAllegienceNone)
					enemies->Delete(numBadGuys); // Ignore anything that isn't actively hostile.
				}
			numBadGuys = enemies->Count();
			if(numBadGuys == 0)
				{
				NextWaveL(); // Create some more.
				}
			}
		CleanupStack::PopAndDestroy(enemies);
		iPlayerPoints = player->Points();
		ScoreTextL();
		UpdatePickupsL();
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::FirstWaveL()
	{
	CGameModel& model = GameDoc().Model();
	CScreenTextBox& box = static_cast<CScreenTextBox&>(model.ScreenBox(iHighScoreBoxIndex));
	box.SetTextL(KNullDesC);

	CGameObject* player = PlayerL();
	__ASSERT_ALWAYS(player, Panic(EPanicModelNoPlayer));
	iFlags &= ~EStatePlayerDead;

	iWave = 0;
	NextWaveL();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::NextWaveL()
	{
	++iWave;
	CGameObject* player = PlayerL();
	if(!player)
		return;
	CGameModel& model = GameDoc().Model();
	CGameObjectController& controller = model.Objects();
	CGameObjectState* baseState = controller.StateL(KGameObjType1_ObjectAsteroidState);
	if(baseState)
		{
		CGameObjectAsteroidState* state = static_cast<CGameObjectAsteroidState*>(baseState);
		const TSize screenSize(model.Screen().Size());
		// Mass.
		TTrigUnit mass(state->MaxMass());
		// Position.
		TCoOrds posDelta;
		posDelta.iY = screenSize.iHeight / (iWave + 1);
		TCoOrds pos;
		pos.iY = posDelta.iY;
		pos.iX = controller.Random(11) - 5;
		pos.iX /= 10;
		pos.iY += (pos.iY * pos.iX);
		pos.iX = 0;
		// Momentum.
		TTrigUnit momentum(controller.Random(21));
		momentum += 10;
		momentum /= 10;
		momentum *= mass;
		// Facing.
		TDirection facing(TDirection::KResolution / 4);
		TInt w = player->Centre().iX.NearestInt();
		w -= (screenSize.iWidth / 2);
		if(w < 0)
			{
			// Player nearest left of screen, wave appears from right of screen.
			facing += (TDirection::KResolution / 2);
			pos.iX = TTrigUnit(screenSize.iWidth) - pos.iX;
			}
		w = controller.Random((TDirection::KResolution / 4) + 1) - (TDirection::KResolution / 8);
		facing += w;
		// Create.
		state->NewAsteroidsL(iWave, pos, posDelta, mass, facing, momentum);
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::ScoreTextL()
	{
	_LIT(KScoreTextFmt,"Wave %d  Score %d");
	iText->Des().Format(KScoreTextFmt, iWave, iPlayerPoints);
	CScreenTextBox& box = static_cast<CScreenTextBox&>(GameDoc().Model().ScreenBox(iTextBoxIndex));
	box.SetTextL(*iText);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::GameOverL()
	{
	CGameModel& model = GameDoc().Model();
	// Update internal flags.
	iFlags |= EStatePlayerDead;
	// Break model<->user link.
	iView->ResetPlayerAddress();
	model.SetPlayerInput(NULL);
	// Update text box.
	CScreenMultilineTextBox& box = static_cast<CScreenMultilineTextBox&>(model.ScreenBox(iHighScoreBoxIndex));
	SetHighScoresScreenTextL(box);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::SaveHighScores()
	{
	if(iCurrentTextBoxLine >= 0)
		{
		TRAPD(ignore, GameDoc().SaveHighScoresL());
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::SetHighScoresScreenTextL(CScreenMultilineTextBox& aTextBox)
	{
	iText->Des().Zero();
	CGameModel& model = GameDoc().Model();
	CGameHighScoreTable* hs = model.HighScores();
	if(hs)
		{
		iCurrentTextBoxLine = hs->FindFirstLower(iPlayerPoints);
		if(iCurrentTextBoxLine >= 0)
			{
			hs->Add(KNullDesC, iPlayerPoints, TUint8(iWave));
			}
		// Set up box.
		const TInt numScores = hs->MaxCount();
		aTextBox.SetNumLinesL(numScores + 2, EFalse);
		FormatHighScoreLinesL(*hs);
		}
	aTextBox.SetTextL(*iText);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::UpdateHighScoreEditL(const TChar& aChar)
	{
	CGameHighScoreTable* hs = GameDoc().Model().HighScores();
	if((hs) && (iCurrentTextBoxLine >= 0))
		{
		TPtr textBuf(iText->Des());
		textBuf.Copy(hs->Name(iCurrentTextBoxLine));
		TInt i = -1;
		if((aChar == TUint(EKeyBackspace)) || (aChar == TUint(EKeyDelete)))
			{
			i = RemoveLastCharFromHighScoreName(textBuf);
			}
		else if(aChar.IsPrint())
			{
			i = AddCharToHighScoreName(textBuf, aChar);
			}
		if(i >= 0)
			{
			hs->Set(iCurrentTextBoxLine, textBuf);
			FormatHighScoreLinesL(*hs);
			static_cast<CScreenTextBox&>(GameDoc().Model().ScreenBox(iHighScoreBoxIndex)).SetTextL(*iText);
			}
		}
	}

///////////////////////////////////////////////////////////////////////////////
TInt CGameAppUi::AddCharToHighScoreName(TDes& aName, TChar aChar)
	{
	TInt i = aName.Locate(TChar(CGameHighScoreTable::KCharUnderscore));
	if(i >= 0)
		aName[i] = TInt16(aChar);
	return i;
	}

///////////////////////////////////////////////////////////////////////////////
TInt CGameAppUi::RemoveLastCharFromHighScoreName(TDes& aName)
	{
	TInt i = aName.Locate(TChar(CGameHighScoreTable::KCharUnderscore)) - 1;
	if(i < 0)
		i = aName.Length() - 1;
	aName[i] = TInt16(CGameHighScoreTable::KCharUnderscore);
	return i;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::FormatHighScoreLinesL(CGameHighScoreTable& aTable)
	{
	_LIT(KDefaultPrefix,"  ");
	_LIT(KLineCursor,">>");
	_LIT(KWaveFmt,  "   wave %2u");
	_LIT(KWaveSpace,"          ");
	_LIT(KPointsFmt,"%11d");
	_LIT(KNewLine,"\n");
	_LIT(KHighScoresText,  "       H i g h   S c o r e s");
	_LIT(KInstructionsText,"   [enter] Play   [escape] Quit");
	const TUint16 KCharTextCursor = '|';
	TPtr textBuf(iText->Des());
	textBuf.Zero();
	const TInt numScores = aTable.MaxCount();
	textBuf.Append(KHighScoresText);
	for(TInt c=0; c<numScores; c++)
		{
		// Line break.
		textBuf.Append(KNewLine);
		// Score name part.
		TBuf<CGameHighScoreTable::KMaxNameLen> name;
		name.Copy(aTable.Name(c));
		if(c == iCurrentTextBoxLine)
			{
			textBuf.Append(KLineCursor);
			AddCharToHighScoreName(name, TChar(KCharTextCursor));
			}
		else
			textBuf.Append(KDefaultPrefix);
		textBuf.Append(name);
		const TUint w = aTable.Wave(c);
		if(w)
			textBuf.AppendFormat(KWaveFmt, w);
		else
			textBuf.Append(KWaveSpace);
		textBuf.AppendFormat(KPointsFmt, aTable.Points(c));
		}
	textBuf.Append(KNewLine);
	textBuf.Append(KInstructionsText);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::AsyncCommandL(TInt aCommandId)
	{
	if(!iAsyncCommand)
		{
		iAsyncCommand = new(ELeave) CAsyncCommand;
		}
	else
		{
		iAsyncCommand->Cancel();
		}
	iAsyncCommand->QueueCommand(aCommandId, *this);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::CleanupRemoveBusy(TAny*)
	{
	CEikonEnv::Static()->BusyMsgCancel();
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// CGameAppUi::CAsyncCommand
///////////////////////////////////////////////////////////////////////////////
CGameAppUi::CAsyncCommand::CAsyncCommand()
: CActive(100)
	{
	CActiveScheduler::Add(this);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::CAsyncCommand::QueueCommand(TInt aCommandId, MEikCommandObserver& aAppUi)
	{
	iCommandId = aCommandId;
	iAppUi = &aAppUi;
	SetActive();
	iStatus = KRequestPending;
	TRequestStatus* status = &iStatus;
	User::RequestComplete(status, KErrNone);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::CAsyncCommand::DoCancel()
	{
	iCommandId = EEikCmdCanceled;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppUi::CAsyncCommand::RunL()
	{
	if( (iAppUi) && (iStatus.Int() == KErrNone) )
		{
		iAppUi->ProcessCommandL(iCommandId);
		}
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
