///////////////////////////////////////////////////////////////////////////////
//
// GameAppUi.h
//
///////////////////////////////////////////////////////////////////////////////
#ifndef GAMEAPPUI_H
#define GAMEAPPUI_H

#include <eikappui.h>
#include "GameModelObserver.h"
#include "Messages.h"
#include "ScreenUtility.h"

class CGameAppView;
class CGameDocument;
//
class CGameObject;
class CGameHighScoreTable;
class CScreenMultilineTextBox;
//

///////////////////////////////////////////////////////////////////////////////
// CGameAppUi
///////////////////////////////////////////////////////////////////////////////
class CGameAppUi : public CEikAppUi, public MGameModelObserver
	{
public:
	CGameAppUi();
	virtual ~CGameAppUi();
	virtual void ConstructL();
	virtual void HandleModelChangeL();
	virtual void HandleCommandL(TInt aCommand);
	virtual void OnGameEventL(CGameModel&, MGameModelObserver::TNotification);
	virtual void DynInitMenuPaneL(TInt aResourceId, CEikMenuPane*);
	virtual void HandleForegroundEventL(TBool aForeground);
private:
	CGameDocument& GameDoc() const;
	TBool IsCommandAvailable(TInt aCommand) const;
	void CmdPlayAsteroidsL();
	void CmdPauseResumeL();
private:
	void StartGameL();
	void StopGame();
	void ResumeGameL();
	void UpdateGameStateL();
	void DoHandleForegroundEventL(TBool aForeground);
private:
	void FirstWaveL();
	void NextWaveL();
	CGameObject* PlayerL();
	void DrawScreen();
	void UpdatePickupsL();
	void ScoreTextL();
	void GameOverL();
	void SaveHighScores();
	void SetHighScoresScreenTextL(CScreenMultilineTextBox&);
	TInt AddCharToHighScoreName(TDes& aName, TChar aChar);
	TInt RemoveLastCharFromHighScoreName(TDes& aName);
	void UpdateHighScoreEditL(const TChar& aChar);
	void FormatHighScoreLinesL(CGameHighScoreTable& aTable);
	void SetScreenTextL(const TDesC& aFmtText);
	void AsyncCommandL(TInt aCommandId);
	static void CleanupRemoveBusy(TAny*);
private:
	class CAsyncCommand : public CActive
		{
	public:
		CAsyncCommand();
		void QueueCommand(TInt aCommandId, MEikCommandObserver& aAppUi);
	protected:
		void DoCancel();
		void RunL();
	private:
		TInt iCommandId;
		MEikCommandObserver* iAppUi;
		};
private:
	CGameAppView* iView;
private:
	// Asteroids game state info.
	enum TFlags { EStatePlayerDead=0x01, EStatePaused=0x02, EStateSuspended=0x04 };
	TUint16 iFlags;
	HBufC* iText;
	TInt iWave;
	TInt iPlayerPoints;
	CAsyncCommand* iAsyncCommand;
	TInt iTextBoxIndex;
	TInt iHighScoreBoxIndex;
	TInt iPowerupsBoxIndex;
	TSize iPowerupsBoxRowSize;
	TScreenPenStyle iPowerupsBoxRowPenStyle;
	TInt iCurrentTextBoxLine;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif