///////////////////////////////////////////////////////////////////////////////
//
// GameAppView.cpp
//
///////////////////////////////////////////////////////////////////////////////
#include "GameAppView.h"
#include "GameAppDoc.h"
#include "Screen.h"
#include "ObjectController.h"
#include "GameTypes.hrh"

#include <eikenv.h>
#include <eikappui.h>
#include <eikhkeyt.h>
#include <gulutil.h>


///////////////////////////////////////////////////////////////////////////////
// CGameAppView
///////////////////////////////////////////////////////////////////////////////
CGameAppView::CGameAppView(CGameDocument& aDoc)
: iDoc(aDoc), iPlayerAddress(TGameMessage::KNullAddress)
	{
	}

///////////////////////////////////////////////////////////////////////////////
CGameAppView::~CGameAppView()
	{
	delete iPlayerKeys;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppView::ConstructL(const TRect& aRect)
	{
	CreateWindowL();
	TRect rect(aRect);
	SetRect(rect);
//	iBorder.SetType(TGulBorder::EDeepRaised);
	iBorder.SetType(TGulBorder::ENone);
	rect = iBorder.InnerRect(Rect());
	iDoc.Model().SetScreenSizeL(rect.Size());
	iScreenPos.iX = (aRect.Width() - rect.Width()) / 2;
	iScreenPos.iY = (aRect.Height() - rect.Height()) / 2;

	iPlayerKeys = new(ELeave) CEikHotKeyTable;
	iPlayerKeys->AddItemL(ECmdThrust, EStdKeyUpArrow, 0);
	iPlayerKeys->AddItemL(ECmdStop, EStdKeyBackspace, 0);
	iPlayerKeys->AddItemL(ECmdTurnLeft, EStdKeyLeftArrow, 0);
	iPlayerKeys->AddItemL(ECmdTurnRight, EStdKeyRightArrow, 0);
	iPlayerKeys->AddItemL(ECmdShoot, EStdKeySpace, 0);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppView::RedrawScreen()
	{
//	Window().Invalidate();
	DrawNow();
	}

///////////////////////////////////////////////////////////////////////////////
TKeyResponse CGameAppView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
	{
	TInt cmd = 0;
	MGameModelPlayerInput::TPlayerKeyState* keyState = NULL;
	switch(aType)
		{
	case EEventKeyDown:
		keyState = &iKeyDown;
		break;
	case EEventKeyUp:
		keyState = &iKeyUp;
		break;
	case EEventKey:
		if(aKeyEvent.iCode == EKeyEnter)
			{
			iKeyDown.SetPressed(EPlayerKeyBegin, ETrue);
			}
		else if(aKeyEvent.iCode == EKeyEscape)
			{
			iKeyDown.SetPressed(EPlayerKeyEsc, ETrue);
			}
		else if(iPlayerAddress == TGameMessage::KNullAddress)
			{
			iKeyChar = aKeyEvent.iCode;
			}
	default:
		break;
		}
	if(keyState)
		{
		cmd = iPlayerKeys->CommandIdFromHotKey(aKeyEvent.iScanCode, 0);
		switch(cmd)
			{
		case ECmdThrust:
			keyState->SetPressed(EPlayerKeyThrust, ETrue);
			break;
		case ECmdStop:
			keyState->SetPressed(EPlayerKeyStop, ETrue);
			break;
		case ECmdTurnLeft:
			keyState->SetPressed(EPlayerKeyTurnLeft, ETrue);
			break;
		case ECmdTurnRight:
			keyState->SetPressed(EPlayerKeyTurnRight, ETrue);
			break;
		case ECmdShoot:
			keyState->SetPressed(EPlayerKeyShoot, ETrue);
			break;
		case 0:
		default:
			cmd = 0;
			break;
			}
		}
	return (cmd) ? (EKeyWasConsumed) : (EKeyWasNotConsumed);
	}

///////////////////////////////////////////////////////////////////////////////
TSize CGameAppView::MinimumSize()
	{
	return iDoc.Model().Screen().Size();
	}

///////////////////////////////////////////////////////////////////////////////
TGameMessage::TAddress CGameAppView::PlayerAddressL(CGameModel& aModel)
	{
	if(iPlayerAddress == TGameMessage::KNullAddress)
		{
		CArrayPtrFlat<CGameObject>* players = aModel.Objects().ObjectsByStateLC(KGameObjType1_ObjectPlayerState);
		if(players->Count())
			{
			iPlayerAddress = players->At(0)->MessageAddress(); // TODO: Multiple players?
			}
		CleanupStack::PopAndDestroy(players);
		}
	return iPlayerAddress;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppView::GetPlayerInput(CGameModel&, MGameModelPlayerInput::TPlayerKeyState& aKeyState)
	{
	aKeyState = iKeyDown;
	if(iKeyUp.IsPressed(EPlayerKeyThrust))
		{
		iKeyDown.SetPressed(EPlayerKeyThrust, EFalse);
		iKeyUp.SetPressed(EPlayerKeyThrust, EFalse);
		}
	if(iKeyUp.IsPressed(EPlayerKeyStop))
		{
		iKeyDown.SetPressed(EPlayerKeyStop, EFalse);
		iKeyUp.SetPressed(EPlayerKeyStop, EFalse);
		}
//	if(iKeyUp.IsPressed(EPlayerKeyTurnLeft))
		{
		iKeyDown.SetPressed(EPlayerKeyTurnLeft, EFalse);
		iKeyUp.SetPressed(EPlayerKeyTurnLeft, EFalse);
		}
//	if(iKeyUp.IsPressed(EPlayerKeyTurnRight))
		{
		iKeyDown.SetPressed(EPlayerKeyTurnRight, EFalse);
		iKeyUp.SetPressed(EPlayerKeyTurnRight, EFalse);
		}
	if(iKeyUp.IsPressed(EPlayerKeyShoot))
		{
		iKeyDown.SetPressed(EPlayerKeyShoot, EFalse);
		iKeyUp.SetPressed(EPlayerKeyShoot, EFalse);
		}
	iKeyDown.SetPressed(EPlayerKeyBegin, EFalse);
	iKeyUp.SetPressed(EPlayerKeyBegin, EFalse);
	iKeyDown.SetPressed(EPlayerKeyEsc, EFalse);
	iKeyUp.SetPressed(EPlayerKeyEsc, EFalse);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppView::ResetPlayerInput(CGameModel&)
	{
	iKeyUp.iKeyState = 0U;
	iKeyDown.iKeyState = 0U;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppView::SizeChangedL()
	{
	const TSize screenSize(iDoc.Model().Screen().Size());
	const TRect rect(iBorder.InnerRect(Rect()));
	iScreenPos.iX = (rect.Width() - screenSize.iWidth) / 2;
	iScreenPos.iY = (rect.Height() - screenSize.iHeight) / 2;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppView::Draw(const TRect& aRect) const
	{
	CEikBorderedControl::Draw(aRect);
	CWindowGc& gc = SystemGc();
	iDoc.Model().Screen().Draw(gc, iScreenPos);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameAppView::ScreenRect(TRect& aRect) const
	{
	aRect.SetRect(iScreenPos, iDoc.Model().Screen().Size());
	}

///////////////////////////////////////////////////////////////////////////////
TChar CGameAppView::GetKeyChar()
	{
	const TChar key(iKeyChar);
	iKeyChar = TChar(0U);
	return key;
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
