///////////////////////////////////////////////////////////////////////////////
//
// GameHighScoreTable.cpp
//
//
//
//
///////////////////////////////////////////////////////////////////////////////
#include "GameHighScoreTable.h"
//

///////////////////////////////////////////////////////////////////////////////
// CGameHighScoreTable
///////////////////////////////////////////////////////////////////////////////
CGameHighScoreTable* CGameHighScoreTable::NewL(TInt aMaxScores)
	{ // static
	CGameHighScoreTable* table = new(ELeave) CGameHighScoreTable;
	CleanupStack::PushL(table);
	table->SetMaxCountL(aMaxScores);
	CleanupStack::Pop(table);
	return table;
	}

///////////////////////////////////////////////////////////////////////////////
CGameHighScoreTable* CGameHighScoreTable::NewL(RReadStream& aStream)
	{ // static
	CGameHighScoreTable* table = new(ELeave) CGameHighScoreTable;
	CleanupStack::PushL(table);
	table->ConstructL(aStream);
	CleanupStack::Pop(table);
	return table;
	}

///////////////////////////////////////////////////////////////////////////////
CGameHighScoreTable::~CGameHighScoreTable()
	{
	iScores.Reset();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::ExternalizeL(RWriteStream& aStream) const
	{
	const TInt count = iScores.Count();
	aStream.WriteUint16L(TUint(count));
	for(TInt c=0; c<count; c++)
		{
		const TScore& score = iScores[c];
		aStream.WriteL(score.iName, KMaxNameLen);
		aStream.WriteInt32L(score.iPoints);
		aStream.WriteUint8L(score.iWave);
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::Clear()
	{
	TInt c = iScores.Count();
	while(c--)
		{
		TScore& score = iScores[c];
		score.iName.Zero();
		FormatName(score.iName);
		score.iPoints = 0;
		score.iWave = TUint8(0U);
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::SetMaxCountL(TInt aCount)
	{
	TScore score;
	FormatName(score.iName);
	score.iPoints = 0;
	score.iWave = 0;
	while(aCount < iScores.Count())
		{
		iScores.Remove(iScores.Count() - 1);
		}
	while(aCount > iScores.Count())
		{
		User::LeaveIfError(iScores.Insert(score, 0));
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::Add(const TDesC& aName, TInt32 aPoints, TUint8 aWave)
	{
	if(iScores.Count() <= 0)
		return;
	iScores.Remove(0); // Remove lowest score.
	TScore score;
	score.iName.Copy(aName);
	FormatName(score.iName);
	score.iPoints = aPoints;
	score.iWave = aWave;
	(void)iScores.InsertInOrderAllowRepeats(score, TLinearOrder<TScore>(PointsOrder)); // Won't leave.
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::Set(TInt aIndex, const TDesC& aName)
	{
	ActualIndex(aIndex);
	TScore& score = iScores[aIndex];
	score.iName.Copy(aName);
	FormatName(score.iName);
	}

///////////////////////////////////////////////////////////////////////////////
TInt CGameHighScoreTable::FindFirstLower(TInt32 aPoints) const
	{
	TInt c = iScores.Count(); // Last item has highest score.
	while(c--)
		{
		if(iScores[c].iPoints < aPoints)
			break;
		}
	ActualIndex(c);
	return c;
	}

///////////////////////////////////////////////////////////////////////////////
TInt CGameHighScoreTable::MaxCount() const
	{
	return iScores.Count();
	}

///////////////////////////////////////////////////////////////////////////////
const TDesC& CGameHighScoreTable::Name(TInt aIndex) const
	{
	ActualIndex(aIndex);
	return iScores[aIndex].iName;
	}

///////////////////////////////////////////////////////////////////////////////
TInt32 CGameHighScoreTable::Points(TInt aIndex) const
	{
	ActualIndex(aIndex);
	return iScores[aIndex].iPoints;
	}

///////////////////////////////////////////////////////////////////////////////
TUint32 CGameHighScoreTable::Wave(TInt aIndex) const
	{
	ActualIndex(aIndex);
	return iScores[aIndex].iWave;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::ConstructL(RReadStream& aStream)
	{
	const TInt count = TInt(aStream.ReadUint16L());
	SetMaxCountL(count);
	for(TInt c=0; c<count; c++)
		{
		TScore& score = iScores[c];
		aStream.ReadL(score.iName, KMaxNameLen);
		score.iPoints = aStream.ReadInt32L();
		score.iWave = aStream.ReadUint8L();
		}
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::FormatName(TName& aName)
	{
	const TInt l = aName.MaxLength() - aName.Length();
	aName.AppendFill(TChar((TUint16)KCharUnderscore), l);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameHighScoreTable::ActualIndex(TInt& aIndex) const
	{
	if(aIndex >= 0)
		aIndex = (iScores.Count() - 1) - aIndex;
	}

///////////////////////////////////////////////////////////////////////////////
TInt CGameHighScoreTable::PointsOrder(const TScore& aLeft, const TScore& aRight)
	{ // static
	return (aLeft.iPoints - aRight.iPoints);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
