///////////////////////////////////////////////////////////////////////////////
//
// ObjectEmitter.h
//
//
// 
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef OBJECTEMITTER_H
#define OBJECTEMITTER_H

#include "Objects.h"


///////////////////////////////////////////////////////////////////////////////
// TGameObjectEmitter
///////////////////////////////////////////////////////////////////////////////
// Class for creating new objects dynamically.
// The new object is given the supplied motion, but is otherwise initialised
// according to the data members of this class.
///////////////////////////////////////////////////////////////////////////////
class TGameObjectEmitter
	{
public:
	typedef CGameObjectState::TId TStateId;
public:
	TGameObjectEmitter();
	virtual void EmitL(CGameObjectController& aController, const TMotionPhysics& aMotion);
public:
	void InternalizeL(TResourceReader&);
	void InternalizeL(RReadStream&);
	void ExternalizeL(RWriteStream&) const;
protected:
	virtual void InitTemplateL(CGameObject::TTemplate& aTemplate);
	void EmitObjectL(CGameObject::TTemplate& aTemplate, const TMotionPhysics& aMotion);
	void EmitObjectL(CGameObject::TTemplate& aTemplate);
public:
	TSpriteId iSprite;
	TStateId  iState;
	TInt16 iAllegience;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// TGameFixedObjectEmitter
///////////////////////////////////////////////////////////////////////////////
// Object emitter for emitting objects that given a parent reference object are
// created with TMotionPhysics values determined by this class's data members.
///////////////////////////////////////////////////////////////////////////////
class TGameFixedObjectEmitter : public TGameObjectEmitter
	{
public:
	TGameFixedObjectEmitter();
	void EmitChildL(const CGameObject& aEmitterObject);
	void EmitChildrenL(TInt aNumObjects, const CGameObject& aEmitterObject);
public:
	void InternalizeL(TResourceReader&);
	void InternalizeL(RReadStream&);
	void ExternalizeL(RWriteStream&) const;
protected:
	virtual void InitTemplateL(CGameObject::TTemplate& aTemplate);
	void CalculatePositionL(CGameObject::TTemplate&);
public:
	TTrigUnit iSpeed;
	TTrigUnit iMass;
	TDirection iFacingOffset;
	TDirection iSpreadAngle;
private:
	TDirection iFacing;
	const CGameObject* iEmitterObject;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
