///////////////////////////////////////////////////////////////////////////////
//
// PlayerStates.h
//
///////////////////////////////////////////////////////////////////////////////
#ifndef PLAYERSTATES_H
#define PLAYERSTATES_H

#include "ObjectStates.h"


///////////////////////////////////////////////////////////////////////////////
// CGameObjectPlayerState
///////////////////////////////////////////////////////////////////////////////
// Messages Handled:
//  KMsgId_ObjectUpdate
//  KMsgId_ObjectCollision
//  KMsgId_ObjectThrust
//  KMsgId_ObjectStop
//  KMsgId_ObjectRotate
//  KMsgId_ObjectShoot
//  KMsgId_ObjectChangeScore
//
///////////////////////////////////////////////////////////////////////////////
class CGameObjectPlayerState : public CGameObjectMotiveSolidState
	{
public:
	CGameObjectPlayerState(CGameObjectController& aController);
	virtual TUidType Type() const;
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual TBool TestCollision(const CGameObject&, TInt aAllegience);
public:
	inline TInt NumShots() const { return iNumShots; }
	inline void SetNumShots(TInt aNumShots) { iNumShots = TUint8(aNumShots); }
	inline TGameFixedObjectEmitter& Weapon() { return iWeapon; }
	inline TGameFixedObjectEmitter& Exhaust() { return iExhaust; }
protected:
	virtual TGameMessage::TResponse OnEnterL(const TGameMessage&);
	virtual TGameMessage::TResponse OnUpdateL(const TGameMessage&);
	virtual TGameMessage::TResponse OnMessageL(const TGameMessage&);
	virtual TGameMessage::TResponse OnCollisionL(const TGameMessage&);
protected:
	TGameMessage::TResponse HandleThrustL(CGameObject& aObject, const TGameMessage&);
	TGameMessage::TResponse HandleRotate(CGameObject& aObject, const TGameMessage&);
	TGameMessage::TResponse HandleShootL(CGameObject& aObject, const TGameMessage&);
	TGameMessage::TResponse HandleScorePoints(CGameObject&, const TGameMessage&);
	TGameMessage::TResponse ActivatePowerupL(CGameObject&, const TGameMessage&, TBool aActive);
	TGameMessage::TResponse HandlePowerupL(CGameObject&, const TGameMessage&);
	TGameMessage::TResponse HandlePowerupExpiryL(CGameObject&, const TGameMessage&);
protected:
	class TStateDataBuf
		{
	public:
		enum TFlags
			{
			EFlagShieldUp = 0x01
			};
	public:
		inline TBool IsShieldUp() const { return iFlags & TUint8(EFlagShieldUp); }
		inline void RaiseShield()       { iFlags |= TUint8(EFlagShieldUp); }
		inline void LowerShield()       { iFlags &= TUint8(~EFlagShieldUp); }
	public:
		TUint8 iReloadStatus;
		TUint8 iReloadTime;
		TUint8 iNumShots;
		TUint8 iFlags;
		TUint8 iShieldDuration;
		};
	void ObjectStateDataBuf(const CGameObject&, TStateDataBuf&) const;
	void SetObjectStateDataBufL(CGameObject&, const TStateDataBuf&) const;
protected:
	TGameFixedObjectEmitter iWeapon;
	TGameFixedObjectEmitter iExhaust;
	TUint8 iNumShots;
	TUint8 iReloadTime;
	TUint8 iShieldDuration;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// CGameObjectPlayerPickupState
///////////////////////////////////////////////////////////////////////////////
class CGameObjectPlayerPickupState : public CGameObjectMotiveSolidState
	{
public:
	CGameObjectPlayerPickupState(CGameObjectController& aController);
	virtual TUidType Type() const;
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual TBool TestCollision(const CGameObject&, TInt aAllegience);
public:
	void CreateNewL(const CGameObject& aParent);
	void CreateNewL(const TCoOrds& aPos);
	inline TSpriteId SpriteId() const { return iEmitter.iSprite; }
protected:
	virtual TGameMessage::TResponse OnEnterL(const TGameMessage& aMessage);
	virtual TGameMessage::TResponse OnCollisionL(const TGameMessage&);
protected:
	TGameMessage::TId iMsgId;
	TBuf8<TGameMessage::KMaxParamLen> iMsgParams;
	TInt16 iPoints;
	TTrigUnit iSpeed;
	TUint16 iSpeedRange;
	TTrigUnit iSpeedDeviation;
	TGameFixedObjectEmitter iEmitter;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
