///////////////////////////////////////////////////////////////////////////////
//
// State.cpp
//
//
///////////////////////////////////////////////////////////////////////////////
#include "State.h"
#include "ObjectController.h"
#include "GameFactory.h"
#include "GameTypes.hrh"
#include "Game.pan"
#include <s32strm.h>
#include <barsread.h>

const TUid KUid0ObjectState          = { KGameObjType0_ObjectState };
const TUid KUid1ObjectStateContainer = { KGameObjType1_ObjectStateContainer };


///////////////////////////////////////////////////////////////////////////////
// CGameObjectState
///////////////////////////////////////////////////////////////////////////////
CGameObjectState::CGameObjectState(CGameObjectController& aController, CGameObjectState::TId aId)
: iController(aController),
  iId(aId)
	{
	}

///////////////////////////////////////////////////////////////////////////////
TUidType CGameObjectState::Type() const
	{
	return TUidType(KUid0ObjectState, TUid::Uid(iId), TUid::Uid(iId));
	}

///////////////////////////////////////////////////////////////////////////////
void CGameObjectState::InternalizeL(TResourceReader& aReader)
	{
	iId = aReader.ReadInt32();
	iExitState = aReader.ReadInt32();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameObjectState::InternalizeL(RReadStream& aStream)
	{
	iId = aStream.ReadInt32L();
	iExitState = aStream.ReadInt32L();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameObjectState::ExternalizeL(RWriteStream& aStream) const
	{
	aStream.WriteInt32L(iId);
	aStream.WriteInt32L(iExitState);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameObjectState::SetId(TInt32)
	{
	}

///////////////////////////////////////////////////////////////////////////////
TBool CGameObjectState::TestCollision(const CGameObject& aObject, TInt aAllegience)
	{
	const TInt allegience = aObject.Allegience();
	return( (allegience != KAllegienceNone) && (aAllegience != KAllegienceNone) && (allegience != aAllegience) );
	}

///////////////////////////////////////////////////////////////////////////////
TGameMessage::TResponse CGameObjectState::HandleMessageL(const TGameMessage& aMessage)
	{
	TGameMessage::TResponse ret = KGameMessageNotHandled;
	switch(aMessage.iId)
		{
		case KMsgId_ObjectEnterState:
			ret = OnEnterL(aMessage);
			break;
		case KMsgId_ObjectUpdate:
			ret = OnUpdateL(aMessage);
			break;
		case KMsgId_ObjectExitState:
			ret = OnExitL(aMessage);
			break;
		default:
			ret = OnMessageL(aMessage);
			break;
		}
	if(ret != KGameMessageNotHandled)
		ObjectL(aMessage).StateData().MessageL(iController, aMessage);
	return ret;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameObjectState::SwitchToState(CGameObject& aObject, CGameObjectState::TId aState)
	{
	aObject.StateData().RequestStateChange(aState);
	}

///////////////////////////////////////////////////////////////////////////////
CGameObject& CGameObjectState::ObjectL(const TGameMessage& aMessage, TMsgObject aObject)
	{
	const TGameMessage::TAddress& a = (aObject==ERecipient) ? (aMessage.iRecipient) : (aMessage.iSender);
	CGameObject* o = iController.ObjectL(a);
	// Will not be NULL since message has been routed to us.
	__ASSERT_DEBUG(o, Panic(EPanicStateUnknownObject));
	return *o;
	}

///////////////////////////////////////////////////////////////////////////////
TGameMessage::TResponse CGameObjectState::OnEnterL(const TGameMessage&)
	{
	return 1;
	}

///////////////////////////////////////////////////////////////////////////////
TGameMessage::TResponse CGameObjectState::OnUpdateL(const TGameMessage&)
	{
	return 1;
	}

///////////////////////////////////////////////////////////////////////////////
TGameMessage::TResponse CGameObjectState::OnExitL(const TGameMessage&)
	{
	return 1;
	}

///////////////////////////////////////////////////////////////////////////////
TGameMessage::TResponse CGameObjectState::OnMessageL(const TGameMessage&)
	{
	return 0;
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
// CGameObjectStateContainer
///////////////////////////////////////////////////////////////////////////////
CGameObjectStateContainer::CGameObjectStateContainer(CGameObjectController& aController)
: iController(aController)
	{
	}

///////////////////////////////////////////////////////////////////////////////
TUidType CGameObjectStateContainer::Type() const
	{
	return TUidType(KUid0ObjectState, KUid1ObjectStateContainer);
	}

///////////////////////////////////////////////////////////////////////////////
MGameStreamable* CGameObjectStateContainer::NewByTypeLC(const TUidType& aType)
	{
	return GameFactory::NewObjectStateByTypeLC(aType, iController);
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
