///////////////////////////////////////////////////////////////////////////////
//
// State.h
//
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef STATE_H
#define STATE_H

#include "GameContainer.h"
#include "GameTypes.hrh"
#include "Messages.h"
class CGameObjectController;
class CGameObject;


///////////////////////////////////////////////////////////////////////////////
// CGameObjectState
///////////////////////////////////////////////////////////////////////////////
// Object 'state'. Derived classes implement specific behaviour.
// Each object has a state, identified by a unique ID. Each unique ID maps to
// a single state object instance. All objects in the same state share the same
// state object instance. Hence all object instance specific state data is held
// in CGameObject.
///////////////////////////////////////////////////////////////////////////////
class CGameObjectState : public CBase, public MGameStreamable
	{
public:
	typedef TInt TId;
	enum TMsgObject
		{
		ERecipient,
		ESender
		};
public:
	CGameObjectState(CGameObjectController& aController, TId aId);
	inline TId Id() const { return iId; }
	virtual TUidType Type() const;
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual void SetId(TInt32);

	virtual TBool TestCollision(const CGameObject&, TInt aAllegience);
	TGameMessage::TResponse HandleMessageL(const TGameMessage&);
	// Virtual message handling.
	virtual TGameMessage::TResponse OnEnterL(const TGameMessage&);
	virtual TGameMessage::TResponse OnUpdateL(const TGameMessage&);
	virtual TGameMessage::TResponse OnExitL(const TGameMessage&);
	virtual TGameMessage::TResponse OnMessageL(const TGameMessage&);

protected:
	inline void Exit(CGameObject& aObj) { SwitchToState(aObj, iExitState); }
	void SwitchToState(CGameObject&, TId aState);
	CGameObject& ObjectL(const TGameMessage&, TMsgObject aObject=ERecipient);
	
protected:
	CGameObjectController& iController;
	TId iId;
	TId iExitState;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

const CGameObjectState::TId KStateIdNull   = 0;
const CGameObjectState::TId KStateIdDead   = CGameObjectState::TId(KGameObjType1_ObjectDeadState);

///////////////////////////////////////////////////////////////////////////////
// CGameObjectStateContainer
///////////////////////////////////////////////////////////////////////////////
// Container for object state instances.
///////////////////////////////////////////////////////////////////////////////
class CGameObjectStateContainer : public CGameContainer<CGameObjectState>
	{
public:
	CGameObjectStateContainer(CGameObjectController& aController);
	virtual TUidType Type() const;
protected:
	virtual MGameStreamable* NewByTypeLC(const TUidType& aType);
protected:
	CGameObjectController& iController;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif
