///////////////////////////////////////////////////////////////////////////////
//
// SpriteBitmap.h
//
//
// Bitmapped sprite classes.
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef SPRITEBITMAP_H
#define SPRITEBITMAP_H

#include "SpriteTable.h"
#include "Screen.h"
class RWsSession;
class CWsBitmap;


///////////////////////////////////////////////////////////////////////////////
// CBitmapSpriteFrame
///////////////////////////////////////////////////////////////////////////////
class CBitmapSpriteFrame : public CBase
	{
public:
	typedef CWsBitmap CBitmap; // Determines the CFbsBitmap class type used.
public:
	CBitmapSpriteFrame(RWsSession&);
	virtual ~CBitmapSpriteFrame();
	void ResetL();
	void LoadL(CWsScreenDevice&, const TDesC& aFile, TInt aFrame, TInt aMask);
	void InternalizeL(RReadStream&);
	void ExternalizeL(RWriteStream&) const;
	void Drawer(TBitmapDrawer&) const;
	TSize Size() const;
private:
	void LoadAndMapBitmapL(CBitmap*&, CWsScreenDevice&, const TDesC&, TInt);
private:
	RWsSession& iWs;
	CBitmap* iBitmap;
	CBitmap* iMask;
	};

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
// CGameBitmapSprite
///////////////////////////////////////////////////////////////////////////////
class CGameBitmapSprite : public CBase, public MGameSpriteObject
	{
public:
	CGameBitmapSprite(RWsSession&, CWsScreenDevice&);
	virtual ~CGameBitmapSprite();
	void ResetL();

	virtual TUidType Type() const;
	virtual void InternalizeL(TResourceReader&);
	virtual void InternalizeL(RReadStream&);
	virtual void ExternalizeL(RWriteStream&) const;
	virtual void SetId(TInt32);
	virtual TInt FrameCount(CGameSpriteData* aSpriteData) const;
	virtual TSize FrameSize(TInt aFrame, CGameSpriteData* aSpriteData) const;
	virtual void DrawFrame(TInt aFrame, CBitmapContext& aGc, const TPoint& aPos, CGameSpriteData* aSpriteData, MMotionBounds* aBounds) const;

	inline void AppendFrameL(CBitmapSpriteFrame*);
	inline void DeleteFrame(TInt aFrame);

protected:
	void ResolveMbmNameL(TParse&, const TDesC& aMbmNameAndPath);
protected:
	RWsSession& iWs;
	CWsScreenDevice& iScreenDevice;
	CArrayPtr<CBitmapSpriteFrame>* iFrames;
	};

///////////////////////////////////////////////////////////////////////////////
inline void CGameBitmapSprite::AppendFrameL(CBitmapSpriteFrame* aFrame)
	{ iFrames->AppendL(aFrame); }
inline void CGameBitmapSprite::DeleteFrame(TInt aFrame)
	{ delete iFrames->At(aFrame); iFrames->Delete(aFrame); }

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
#endif

