///////////////////////////////////////////////////////////////////////////////
//
// SpriteParticle.cpp
//
//
// Particle sprite classes.
//
//
///////////////////////////////////////////////////////////////////////////////
#include "SpriteParticle.h"
#include "ScreenUtility.h"
#include "GameTypes.hrh"
#include <s32strm.h>
#include <barsread.h>

const TUid KUidSprite         = { KGameObjType0_Sprite };
const TUid KUidParticleSprite = { KGameObjType1_SpriteParticle };

///////////////////////////////////////////////////////////////////////////////
// CGameParticleSprite
///////////////////////////////////////////////////////////////////////////////
CGameParticleSprite::CGameParticleSprite(TRgb aColor)
: iColorFrames(aColor)
	{
	}

///////////////////////////////////////////////////////////////////////////////
void CGameParticleSprite::SetFrames(const TFadedColor& aFrames)
	{
	iColorFrames = aFrames;
	}

///////////////////////////////////////////////////////////////////////////////
TUidType CGameParticleSprite::Type() const
	{
	return TUidType(KUidSprite, KUidParticleSprite);	
	}

///////////////////////////////////////////////////////////////////////////////
void CGameParticleSprite::SetId(TInt32)
	{
	}

///////////////////////////////////////////////////////////////////////////////
void CGameParticleSprite::InternalizeL(TResourceReader& aReader)
	{
	iColorFrames.InternalizeL(aReader);
	}

///////////////////////////////////////////////////////////////////////////////
void CGameParticleSprite::InternalizeL(RReadStream& aStream)
	{
	aStream >> iColorFrames;
	}

///////////////////////////////////////////////////////////////////////////////
void CGameParticleSprite::ExternalizeL(RWriteStream& aStream) const
	{
	aStream << iColorFrames;
	}

///////////////////////////////////////////////////////////////////////////////
TInt CGameParticleSprite::FrameCount(CGameSpriteData*) const
	{
	return iColorFrames.Steps();
	}

///////////////////////////////////////////////////////////////////////////////
TSize CGameParticleSprite::FrameSize(TInt, CGameSpriteData* aSpriteData) const
	{
	if(!aSpriteData)
		{
		return TSize(2, 2);
		}
	const TRect r(aSpriteData->Bounds());
	return r.Size();
	}

///////////////////////////////////////////////////////////////////////////////
void CGameParticleSprite::DrawFrame(TInt aFrame, CBitmapContext& aGc, const TPoint& aPos, CGameSpriteData* aSpriteData, MMotionBounds* aBounds) const
	{
	const TRgb c(iColorFrames.Color(aFrame));
	aGc.Reset();
	aGc.SetPenColor(c);
	aGc.SetPenSize(TSize(2,2));
	aGc.SetPenStyle(CGraphicsContext::ESolidPen);
	aGc.SetDrawMode(CGraphicsContext::EDrawModeXOR);
	if(aSpriteData)
		{
		aSpriteData->PlotPoints(aGc, aPos, aBounds);
		}
	else
		{
		aGc.Plot(aPos);
		}
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
