///////////////////////////////////////////////////////////////////////////////
//
// SpritePolygon.cpp
//
//
// Polygon sprite classes.
//
//
///////////////////////////////////////////////////////////////////////////////
#include "SpritePolygon.h"
#include "GameResourceUtils.h"
#include "Motion.h"
#include "GameTypes.hrh"
#include <s32strm.h>
#include <barsread.h>

const TUid KUidSprite       = { KGameObjType0_Sprite };
const TUid KUidPolygonSprite = { KGameObjType1_SpritePolygon };

///////////////////////////////////////////////////////////////////////////////
// CGamePolygonSprite
///////////////////////////////////////////////////////////////////////////////
CGamePolygonSprite::CGamePolygonSprite()
	{
	}

///////////////////////////////////////////////////////////////////////////////
void CGamePolygonSprite::SetLineStyle(CGraphicsContext::TPenStyle aLineStyle)
	{
	iLineStyle = aLineStyle;
	}

///////////////////////////////////////////////////////////////////////////////
TUidType CGamePolygonSprite::Type() const
	{
	return TUidType(KUidSprite, KUidPolygonSprite);
	}

///////////////////////////////////////////////////////////////////////////////
void CGamePolygonSprite::InternalizeL(TResourceReader& aReader)
	{
	iLineStyle = CGraphicsContext::TPenStyle(aReader.ReadUint8());
	iDefaultLineColor.InternalizeL(aReader);
	}

///////////////////////////////////////////////////////////////////////////////
void CGamePolygonSprite::InternalizeL(RReadStream& aStream)
	{
	iLineStyle = CGraphicsContext::TPenStyle(aStream.ReadUint8L());
	iDefaultLineColor.InternalizeL(aStream);
	}

///////////////////////////////////////////////////////////////////////////////
void CGamePolygonSprite::ExternalizeL(RWriteStream& aStream) const
	{
	aStream.WriteUint8L((TUint)iLineStyle);
	iDefaultLineColor.ExternalizeL(aStream);
	}

///////////////////////////////////////////////////////////////////////////////
void CGamePolygonSprite::SetId(TInt32)
	{
	}

///////////////////////////////////////////////////////////////////////////////
TInt CGamePolygonSprite::FrameCount(CGameSpriteData*) const
	{
	return 1;
	}

///////////////////////////////////////////////////////////////////////////////
TSize CGamePolygonSprite::FrameSize(TInt, CGameSpriteData* aSpriteData) const
	{
	if(!aSpriteData)
		{
		return TSize(1, 1);
		}
	return aSpriteData->Bounds().Size();
	}

///////////////////////////////////////////////////////////////////////////////
void CGamePolygonSprite::DrawFrame(TInt aFrame, CBitmapContext& aGc, const TPoint& aPos, CGameSpriteData* aSpriteData, MMotionBounds* aBounds) const
	{
	// Offset the vertices.
	if(aSpriteData)
		{
		aGc.Reset();
		aGc.SetBrushStyle(CGraphicsContext::ENullBrush);
		aGc.SetPenStyle(iLineStyle);
		aGc.SetPenSize(TSize(1,1));
		aGc.SetPenColor(iDefaultLineColor.Color(aFrame));
		aSpriteData->PlotPolygon(aGc, aPos, aBounds);
		if(aBounds)
			{
			const TSize extents(aBounds->Extents());
			TRect r(aSpriteData->Bounds());
			r.Move(aPos);
			const TCoOrds pos(aPos);
			TCoOrds coOrds(pos);
			if(r.iTl.iX < 0)
				{
				coOrds.iX += extents.iWidth;
				}
			else if(r.iBr.iX > extents.iWidth)
				{
				coOrds.iX -= extents.iWidth;
				}
			if(coOrds != pos)
				{
				aSpriteData->PlotPolygon(aGc, coOrds, aBounds);
				}
			coOrds = pos;
			if(r.iTl.iY < 0)
				{
				coOrds.iY += extents.iHeight;
				}
			else if(r.iBr.iY > extents.iHeight)
				{
				coOrds.iY -= extents.iHeight;
				}
			if(coOrds != pos)
				{
				aSpriteData->PlotPolygon(aGc, coOrds, aBounds);
				}
			}
		}
	}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
